% 
% [K, M, C] = HeatStiffnessMass (coor)
% 
% Input:
%   [coor] It contains the location of the nodes (1D)
% 
% Output:
%   [K] Derivative matrix of the shape function product:
%       K = Int (dN * dN')
%   [M] Mass matrix of the shape function product:
%       M = Int (N * N')
%   [C] Matrix of the shape function product:
%       M = Int (N * dN')
% 
% 
% 
% Alberto Badias Herbera
%
% Applied Mechanics and Bioengineering.
% Universidad de Zaragoza.
%
% START:      21 Abril 2016
% LAST MODIF: 21 ABRIL 2016
%

function [K, M, C] = HeatStiffnessMass1D (coor)

K = sparse(length(coor),length(coor));
M = sparse(length(coor),length(coor));
C = sparse(length(coor),length(coor));
% 1D Domain, Linear Shape Function, Number of Gauss Points = 2
wg = [1 1]; % Weight of the Integration Points
sg = [-1/sqrt(3) 1/sqrt(3)]; % Position of the Integration Points

for i1=1:length(coor)-1    
    a = coor(i1); % Left coordinate of the 'x'-interval element
    b = coor(i1+1); % Right coordinate of the 'x'-interval element
    L = b - a; % Length of the element in 'x' space
    jcob = L * 0.5; % Jacobian (transformation from 'chi' to 'x' after numerical integration)
    X1 = (a+b)*0.5 + sg(1)*(b-a)*0.5; % First integration point in the 'x' space corresponding 'chi1' coordinate
    X2 = (a+b)*0.5 + sg(2)*(b-a)*0.5; % Second integration point in the 'x' space corresponding 'chi2' coordinate
    
    M(i1:i1+1,i1:i1+1) = M(i1:i1+1,i1:i1+1) + jcob * wg(1) * ([(b-X1);(X1-a)]/ L * [(b-X1);(X1-a)]'/ L) + ...
        jcob * wg(2) * ([(b-X2);(X2-a)]/ L * [(b-X2);(X2-a)]'/ L); % Mass Matrix
    
    K(i1:i1+1,i1:i1+1) = K(i1:i1+1,i1:i1+1) + jcob * wg(1) * ([-1;1]/ L * [-1;1]'/ L) + ...
        jcob * wg(2) * ([-1;1]/ L * [-1;1]'/ L); % Stiffness Matrix
    
    C(i1:i1+1,i1:i1+1) = C(i1:i1+1,i1:i1+1) + jcob * wg(1) * ([(b-X1);(X1-a)]/ L * [-1;1]'/ L) + ...
        jcob * wg(2) * ([(b-X2);(X2-a)]/ L * [-1;1]'/ L); % 'C' Matrix
    
end
   
end