%
% Script made to compute the Transient Heat Transfer equation in a 1D domain
% 
% 
% Alberto Badias Herbera
%
% Applied Mechanics and Bioengineering.
% Universidad de Zaragoza.
%
% UKACM School on Advanced Topics in Computational Mechanics
%
% 14th April 2021
%

clear all; close all; clc;

%
% ALGORITHM VARIABLES
%
num_max_iter = 50; % Max . # of summands for the approach
TOL = 1.0E-4; % Tolerance of PGD
TOLSource = 1.0E-5; % Tolerance of the Source Descomposition

% 
% GEOMETRY
% 
Xini = 0;
Xend = pi; 
Tini = 0;
Tend = 1;
nnX = 151; 
nnT = 201;
coorX = linspace (Xini,Xend,nnX);
coorT = linspace (Tini,Tend,nnT);
dt = (Tend-Tini)/(nnT-1);

%
% THERMAL CONSTANTS
%
Kconduct = 0.05;
ro = 1;
Cp = 1;

%
% ALLOCATION OF IMPORTANT MATRICES
%
F = sparse(nnX,1);
G = sparse(nnT,1);

%
% COMPUTING STIFFNESS AND MASS MATRICES ALONG EACH DIRECTION
%
[KX,MX,CX] = HeatStiffnessMass1D(coorX);
[KT,MT,CT] = HeatStiffnessMass1DTime(coorT, dt);

%
% SOURCE TERM IN A SEPARATED WAY
%
A = 100;
L = 0.5;
xi = 2*pi/7;
xf = 5*pi/7;
ti = 0.2;
tf = 0.7;


SourceG=zeros(nnX,1);
DSource=[];

% Source definition:
for t=Tini:1/(nnT-1):Tend
    position=0;
    for xflux=Xini:pi/(nnX-1):Xend
        position=position+1;
        x0=xi+(xf-xi)/(tf-ti)*(t-ti);
        if (t>ti) && (t<tf) && ((xflux-x0)<(L/2)) && ((xflux-x0)>(-L/2))
            SourceG(position)=A*cos(pi/L*(xflux-x0));
        else
            SourceG(position)=0;
        end
    end
    DSource=[DSource, SourceG];
end

% Source in separated variables:
[FR1,FR2] = OwnFuenteDiscreta(coorX,coorT,TOLSource, DSource);

% Source terms with the mass matrix premultiplied:
for k1 =1: size (FR1,2)
    ah = FR1(:,k1);    
    VX(:,k1) = MX * ah;
end
for k2 =1: size (FR2 ,2)
    bh = FR2(:,k2);
    VY(:,k2) = MT * bh;
end


%
% BOUNDARY CONDITIONS
%
BCX = [1 numel(coorX)];
BCY = 1;
CCX = setxor (BCX,1:numel(coorX))';
CCY = setxor (BCY,1:numel(coorT))';
    
%
% ENRICHMENT OF THE APPROXIMATION , LOOKING FOR R AND S
%
num_iter = 0; iter = 0; Aprt = 0; Error_iter = 1.0;
while Error_iter >TOL && num_iter < num_max_iter
    num_iter = num_iter + 1;

    % Initial guess for R and S.
    % It works equally well by choosing something random.
    R0 = zeros(numel(coorX),1);
    S0 = ones(numel(coorT),1);
    % We impose that initial guess for functions R and S verify
    % homogeneous essential boundary conditions .
    R0(BCX) = 0;
    S0(BCY) = 0;

    %
    % ENRICHMENT STEP
    %
    [R, S, iter(num_iter)] = HeatEnrichment (KX, KT, MX, MT, VX, VY, CX, CT, R0, S0, F, G, CCX, CCY, BCX, BCY, TOL, Kconduct, ro, Cp);
    
    %
    % UPDATE THE VALUE OF 'F' AND 'G'
    %
    F(:, num_iter) = R;
    G(:, num_iter) = S;
    
    %
    % STOPPING CRITERION
    %
    % One possible criterion is to stop when the norm of the new sum is
    % negligible wrt the pair of functions with the maximum norm
    
    Error_iter = norm(F(:,num_iter)) * norm(G(:,num_iter));
        
    Aprt = max(Aprt,sqrt(Error_iter));
    Error_iter = sqrt(Error_iter)/Aprt ;
    fprintf (1, '%dst   summand  in %d  iterations   with  a  weight  of %f\n' ,...
    num_iter , iter(num_iter), sqrt ( Error_iter ));
end

% num_iter = num_iter - 1;% the last sum was negligible , we discard it.
fprintf (1, 'PGD   Process   exited   normally \n\n');

%
% POST - PROCESSING
%

% Modes plotting:
figure, plot(coorX,F(: ,1: num_iter ));
figure, plot(coorT,G(: ,1: num_iter ));

% Surface plotting:
figure ;
surf ( coorT , coorX , F*G','EdgeColor','none');

